def catx(arr1, arr2, arr3):
 	return [r1 + r2 + r3 for r1, r2, r3 in zip(arr1, arr2, arr3)]

def caty(arr1, arr2, arr3, arr4):
 	return arr1 + arr2 + arr3 + arr4

# apologies for this :D
def shorten_middle(curve):
	new = [[curve[i][j] if j < len(row)//3 or j >= 2*len(row)//3 else
	((curve[i-len(curve)//14][j] if i >= len(curve)//14 else " ") if i < len(curve)//2 else curve[i+len(curve)//14][j] if i < len(curve) - len(curve)//14 else " ")
	for j, c in enumerate(row)] for i, row in enumerate(curve)]
	return new[len(curve)//14:-len(curve)//14]

def empty(curve):
	return [[" "]*len(row) for row in curve]

def filled(curve):
	return [["█"]*len(row) for row in curve]

def upper(curve):
	return [row for i, row in enumerate(curve) if i < 3*len(curve)//4]

def lower(curve):
	return [row for i, row in enumerate(curve) if i >= len(curve)//4]

def right_up(curve):
	return [["█" if j < len(row)//2 else c for j, c in enumerate(row)] if i >= len(curve)//4 else row for i, row in enumerate(curve)]

def left_down(curve):
	return [["█" if j > len(row)//2-1 else c for j, c in enumerate(row)] if i < 3*len(curve)//4 else row for i, row in enumerate(curve)]

def right_down(curve):
	return [["█" if j < len(row)//2 else c for j, c in enumerate(row)] if i < 3*len(curve)//4 else row for i, row in enumerate(curve)]

def left_up(curve):
	return [["█" if j > len(row)//2-1 else c for j, c in enumerate(row)] if i >= len(curve)//4 else row for i, row in enumerate(curve)]


def koch(n):
	if n == 0:
		return [["◢◣"]]
	elif n == 1:
		return [[" ", " ","◢", "◣", " ", " "], ["◥", "█", "█", "█", "█", "◤"],["◢", "█", "█", "█", "█", "◣"], [" ", " ", "◥", "◤", " ", " "]]
	else:
		k = koch(n-1)
		f = filled(k)
		e = empty(k)
		u = catx(e, upper(k), e)
		m1 = catx(left_up(k), f, right_up(k))
		m2 = catx(left_down(k), f, right_down(k))
		l = catx(e, lower(k), e)
		k = caty(u, m1, m2, l)
		k = shorten_middle(k)
	return k

def koch_string(n):
	return "\n".join("".join(row) for row in koch(n))

if __name__ == "__main__":
	print(koch_string(2))



def catx(arr1, arr2, arr3):
    return [r1 + r2 + r3 for r1, r2, r3 in  zip(arr1, arr2, arr3)]

def caty(arr1, arr2, arr3):
    return arr1 + arr2 + arr3

def vicsek(n):
    if n == 0:
        return [["┼"]]
    c = vicsek(n-1)
    l = len(c)
    m = [[" " for _ in range(l)] for _ in range(l)]
    m = catx(m, c, m)
    c = catx(c, c, c)
    return caty(m, c, m)

def vicsek_string(n):
    return "\n".join("".join(s) for s in vicsek(n))

if __name__ == '__main__':
	print(vicsek_string(2))



import frascii
import argparse

def cmd_main():
	fc = argparse.ArgumentDefaultsHelpFormatter
	parser = argparse.ArgumentParser(prog="frascii", description="ASCII visualization of fractals for the terminal.", formatter_class=fc)
	subparsers = parser.add_subparsers(dest="subcommand")

	# Sierpinski carpet sub-command
	sierpinski_parser = subparsers.add_parser("sierpinski_carpet", description="Wikipedia: https://en.wikipedia.org/wiki/Sierpinski_carpet", formatter_class=fc)
	sierpinski_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 7), nargs="?", default=2)
	sierpinski_parser.set_defaults(func=frascii.sierpinski_carpet)

	# Sierpinski triangle sub-command
	sierpinski_parser = subparsers.add_parser("sierpinski_triangle", description="Wikipedia: https://en.wikipedia.org/wiki/Sierpinski_triangle", formatter_class=fc)
	sierpinski_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 10), nargs="?", default=3)
	sierpinski_parser.set_defaults(func=frascii.sierpinski_triangle)

	# Cantor set sub-command
	cantor_parser = subparsers.add_parser("cantor", description="Wikipedia: https://en.wikipedia.org/wiki/Cantor_set", formatter_class=fc)
	cantor_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 7), nargs="?", default=2)
	cantor_parser.set_defaults(func=frascii.cantor)

	# Koch snowflake sub-command
	koch_parser = subparsers.add_parser("koch", description="Wikipedia: https://en.wikipedia.org/wiki/Koch_snowflake", formatter_class=fc)
	koch_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 7), nargs="?", default=2)
	koch_parser.set_defaults(func=frascii.koch)

	# Vicsek fractal sub-command
	vicsek_parser = subparsers.add_parser("vicsek", description="Wikipedia: https://en.wikipedia.org/wiki/Vicsek_fractal", formatter_class=fc)
	vicsek_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 7), nargs="?", default=2)
	vicsek_parser.set_defaults(func=frascii.vicsek)

	# Hilbert curve sub-command
	hilbert_parser = subparsers.add_parser("hilbert_curve", description="Wikipedia: https://en.wikipedia.org/wiki/Hilbert_curve", formatter_class=fc)
	hilbert_parser.add_argument("-n", type=int, help="iterations", choices=range(1, 10), nargs="?", default=3)
	hilbert_parser.set_defaults(func=frascii.hilbert_curve)

	# Peano curve sub-command
	peano_parser = subparsers.add_parser("peano_curve", description="Wikipedia: https://en.wikipedia.org/wiki/Peano_curve", formatter_class=fc)
	peano_parser.add_argument("-n", type=int, help="iterations", choices=range(1, 7), nargs="?", default=2)
	peano_parser.set_defaults(func=frascii.peano_curve)

	# Dragon curve sub-command
	dragon_parser = subparsers.add_parser("dragon_curve", description="Wikipedia: https://en.wikipedia.org/wiki/Dragon_curve", formatter_class=fc)
	dragon_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 18), nargs="?", default=7)
	dragon_parser.set_defaults(func=frascii.dragon_curve)

	# Fibonacci sub-command
	fibonacci_parser = subparsers.add_parser("fibonacci", description="Wikipedia: https://en.wikipedia.org/wiki/Fibonacci_number", formatter_class=fc)
	fibonacci_parser.add_argument("-n", type=int, help="iterations", choices=range(0, 14), nargs="?", default=3)
	fibonacci_parser.set_defaults(func=frascii.fibonacci)

	# Mandelbrot sub-command
	mandelbrot_parser = subparsers.add_parser("mandelbrot", description="Wikipedia: https://en.wikipedia.org/wiki/Mandelbrot_set", formatter_class=fc)
	mandelbrot_parser.add_argument("-x", type=float, help="real part of the images center", nargs="?", default=-0.8)
	mandelbrot_parser.add_argument("-y", type=float, help="imaginary part of the images center", nargs="?", default=0)
	mandelbrot_parser.add_argument("-x_radius", type=int, help="pixels added on left and right of center", nargs="?", default=27)
	mandelbrot_parser.add_argument("-y_radius", type=int, help="pixels added on top and bottom of center", nargs="?", default=12)
	mandelbrot_parser.add_argument("-stepsize", type=float, help="size of a pixel", nargs="?", default=0.05)
	mandelbrot_parser.add_argument("-max_iter", type=int, help="iterations after which z is assumed to be in the set", nargs="?", default=20)
	mandelbrot_parser.set_defaults(func=frascii.mandelbrot)

	# Julia sub-command
	julia_parser = subparsers.add_parser("julia", description="Wikipedia: https://en.wikipedia.org/wiki/Julia_set", formatter_class=fc)
	julia_parser.add_argument("-f", type=str, help="complex function", nargs="?", default="z**2 - 0.4 + 0.6j")
	julia_parser.add_argument("-x", type=float, help="real part of the images center", nargs="?", default=0)
	julia_parser.add_argument("-y", type=float, help="imaginary part of the images center", nargs="?", default=0)
	julia_parser.add_argument("-x_radius", type=int, help="pixels added on left and right of center", nargs="?", default=45)
	julia_parser.add_argument("-y_radius", type=int, help="pixels added on top and bottom of center", nargs="?", default=18)
	julia_parser.add_argument("-stepsize", type=float, help="size of a pixel", nargs="?", default=0.033)
	julia_parser.add_argument("-max_iter", type=int, help="iterations after which z is assumed to be in the set", nargs="?", default=40)
	julia_parser.set_defaults(func=frascii.julia)

	args = parser.parse_args()
	if args.subcommand == None:
		parser.print_help()
		parser.exit()
		return
	func = args.func
	a = {key: value for key, value in args.__dict__.items() if key not in ['func', 'subcommand']}
	result = func(**a)
	print(result)

if __name__ == '__main__':
	cmd_main()



def in_mandelbrot(c, max_iter):
    z = 0
    for i in range(max_iter):
        z = z**2 + c
        if abs(z) >= 2:
            return i
    return max_iter

def mandelbrot_string(x, y, x_radius, y_radius, stepsize, max_iter):
    mset = [[int(4*in_mandelbrot(complex(x + dx * stepsize, y - dy * stepsize * 2), max_iter) / max_iter)
			for dx in range(-x_radius, x_radius+1)] for dy in range(-y_radius, y_radius+1)]
    chars = [' ', '░', '▒', '▓', '█']
    return '\n'.join(''.join(chars[d] for d in l) for l in mset)

if __name__ == '__main__':
	print(mandelbrot_string(-0.8, 0, 27, 12, 0.05, 20))



def rot90(arr, direction):
	m = len(arr)
	n = len(arr[0])
	if direction == 'cw':
		return [[arr[m-1-j][i] for j in range(m)] for i in range(n)]
	elif direction == 'ccw':
		return [[arr[j][n-1-i] for j in range(m)] for i in range(n)]

def fliplr(arr):
	return [row[::-1] for row in arr]

def flipud(arr):
	return [row for row in arr[::-1]]

def catx(arr1, arr2, arr3):
	return [r1 + r2 + r3 for r1, r2, r3 in zip(arr1, arr2, arr3)]

def caty(arr1, arr2, arr3):
	return arr1 + arr2 + arr3

def add_value(arr, value):
    return [[elem + value for elem in row] for row in arr]

def peano_curve(n):
	if n == 1:
		return [[2, 3, 8], [1, 4, 7], [0, 5, 6]]
	hlu = peano_curve(n-1)
	hru = fliplr(hlu)
	hld = flipud(hlu)
	hrd = flipud(hru)
	add = 9 ** (n-1)
	h_up = catx(add_value(hlu, 2*add), add_value(hld, 3*add), add_value(hlu, 8*add))
	h_mid = catx(add_value(hru, 1*add), add_value(hrd, 4*add), add_value(hru, 7*add))
	h_down = catx(hlu, add_value(hld, 5*add), add_value(hlu, 6*add))
	h = caty(h_up, h_mid, h_down)
	return h

def peano_curve_string(iterations):
	arr = peano_curve(iterations)
	out = ""
	n = len(arr)
	for i in range(n):
		for j in range(n):
			if i != 0 and i != n-1 and abs(arr[i-1][j] - arr[i][j]) == 1 and abs(arr[i+1][j] - arr[i][j]) == 1:
				out += "│ "
			elif j != 0 and j != n-1 and abs(arr[i][j-1] - arr[i][j]) == 1 and abs(arr[i][j+1] - arr[i][j]) == 1:
				out += "──"
			elif j != 0 and i != n-1 and abs(arr[i][j-1] - arr[i][j]) == 1 and abs(arr[i+1][j] - arr[i][j]) == 1:
				out += "┐ "
			elif j != 0 and i != 0 and abs(arr[i][j-1] - arr[i][j]) == 1 and abs(arr[i-1][j] - arr[i][j]) == 1:
				out += "┘ "
			elif i != 0 and j != n-1 and abs(arr[i-1][j] - arr[i][j]) == 1 and abs(arr[i][j+1] - arr[i][j]) == 1:
				out += "└─"
			elif i != n-1 and j != n-1 and abs(arr[i+1][j] - arr[i][j]) == 1 and abs(arr[i][j+1] - arr[i][j]) == 1:
				out += "┌─"
			else:
				out += "│ " if iterations % 9 else "──"
			if j == n-1:
				out = out[:-1] + "\n"
	return out[:-1]

if __name__ == '__main__':
	print(peano_curve_string(2))



from frascii.fractals.mandelbrot import mandelbrot_string
from frascii.fractals.julia import julia_string
from frascii.fractals.sierpinski_carpet import sierpinski_carpet_string
from frascii.fractals.sierpinski_triangle import sierpinski_triangle_string
from frascii.fractals.cantor import cantor_string
from frascii.fractals.koch import koch_string
from frascii.fractals.vicsek import vicsek_string
from frascii.fractals.hilbert_curve import hilbert_curve_string
from frascii.fractals.peano_curve import peano_curve_string
from frascii.fractals.dragon_curve import dragon_curve_string
from frascii.fractals.fibonacci import fibonacci_string

__version__ = '2.1'

def mandelbrot(x, y, x_radius, y_radius, stepsize, max_iter):
	return mandelbrot_string(x, y, x_radius, y_radius, stepsize, max_iter)

def julia(f, x, y, x_radius, y_radius, stepsize, max_iter):
	return julia_string(f, x, y, x_radius, y_radius, stepsize, max_iter)

def sierpinski_carpet(n):
	return sierpinski_carpet_string(n)

def cantor(n):
	return cantor_string(n)

def vicsek(n):
	return vicsek_string(n)

def koch(n):
	return koch_string(n)

def sierpinski_triangle(n):
	return sierpinski_triangle_string(n)

def hilbert_curve(n):
	return hilbert_curve_string(n)

def peano_curve(n):
	return peano_curve_string(n)

def dragon_curve(n):
	return dragon_curve_string(n)

def fibonacci(n):
	return fibonacci_string(n)



from setuptools import setup, find_packages
from frascii import __version__

setup(	
		name='frascii', version=__version__,
		url='https://github.com/LuggiStruggi/Frascii.git',
		author='Lukas König',
		author_email='lukasmkoenig@gmx.net',
		packages=find_packages(),
		entry_points={'console_scripts': ['frascii=frascii.commands:cmd_main']},
		decription="DESCRIPTION"
	)



def catx(arr1, arr2):
    return [r1 + r2 for r1, r2 in zip(arr1, arr2)]

def caty(arr1, arr2):
    return arr1 + arr2

def reducer(arr):
    return [r[:-1] for r in arr]

def reducel(arr):
    return [r[1:] for r in arr]

def reducet(arr):
    return arr[1:]

def reduceb(arr):
    return arr[:-1]

def square(n):
    top = [["┌"] + ["─"]*(2*n+1) + ["┐"]]
    mid = [["│"] + [" "]*(2*n+1) + ["│"]]*n
    bot = [["└"] + ["─"]*(2*n+1) + ["┘"]]
    return top + mid + bot

def fibonacci(n):
    if n == 0:
        return square(0)
    f = fibonacci(n-1)
    if n % 4 == 3:
        l = len(f)-2
        f[0][0] = "┬"
        f[-1][0] = "┴"
        return catx(reducer(square(l)), f)
    elif n % 4 == 0:
        f[-1][0] = "├"
        f[-1][-1] = "┤"
        w = len(f[0])//2 - 1
        return caty(f, reducet(square(w)))
    elif n % 4 == 1:
        l = len(f)-2
        f[0][-1] = "┬"
        f[-1][-1] = "┴"
        return catx(f, reducel(square(l)))
    elif n % 4 == 2:
        f[0][0] = "├"
        f[0][-1] = "┤"
        w = len(f[0])//2 - 1
        return caty(reduceb(square(w)), f)

def fibonacci_string(n):
    return "\n".join("".join(row) for row in fibonacci(n))

if __name__ == '__main__':
	print(fibonacci_string(3))
